from django.urls import path

from django_spire.knowledge.entry.views import template_views

app_name = 'template'

urlpatterns = [
    path('files/<int:collection_pk>', template_views.file_list_view, name='file_list'),
]



from __future__ import annotations

import html
import re

import marko

from typing import TYPE_CHECKING

from marko.element import Element
from marko.block import Heading, List, ListItem

from django_spire.knowledge.entry.version.converters.converter import \
    BaseConverter
from django_spire.knowledge.entry.version.block import models

if TYPE_CHECKING:
    from django_spire.knowledge.entry.version.models import EntryVersion
    from django_spire.file.models import File
    from marko.block import BlockElement


class MarkdownConverter(BaseConverter):
    """Converts Markdown content to a list of EntryVersionBlocks using Marko.

    For more info on Marko:
    https://marko-py.readthedocs.io/en/latest/api.html#marko.block.BlockElement
    """
    def __init__(self, entry_version: EntryVersion):
        super().__init__(entry_version)
        self._order = 0

    def convert_file_to_blocks(self, file: File) -> list[models.EntryVersionBlock]:
        with open(file.file.path, 'r') as f:
            return self.convert_markdown_to_blocks(f.read())

    def _convert_heading_block(
            self,
            marko_block: Heading,
            order: int
    ) -> models.EntryVersionBlock:
        heading_type = (
            models.BlockTypeChoices.HEADING
            if marko_block.level == 1
            else models.BlockTypeChoices.SUB_HEADING
        )

        return models.EntryVersionBlock.services.factory.create_null_block(
            entry_version=self.entry_version,
            block_type=heading_type,
            order=order,
            value=self._get_marko_text_content(marko_block),
        )

    def _convert_list_block(
            self,
            marko_block: List | ListItem | Element,
            bullet: str,
            indent_level: int,
            ordered: bool,
    ) -> list[models.EntryVersionBlock]:
        if isinstance(marko_block.children, str):
            list_item_block = models.EntryVersionBlock.services.factory.create_null_block(
                entry_version=self.entry_version,
                block_type=models.BlockTypeChoices.LIST_ITEM,
                order=self._order,
                value=self._get_marko_text_content(marko_block),
                bullet=bullet,
                indent_level=indent_level,
                ordered=ordered,
            )
            self._order += 1
            return [list_item_block]

        if isinstance(marko_block, ListItem):
            indent_level += 1

        blocks = []
        for child in marko_block.children:
            blocks.extend(
                self._convert_list_block(
                    marko_block=child,
                    bullet=bullet,
                    indent_level=indent_level,
                    ordered=ordered,
                )
            )

            if isinstance(child, ListItem) and bullet.endswith('.'):
                bullet = bullet.rstrip('.')
                bullet = str(int(bullet) + 1) + '.'

        return blocks

    def convert_markdown_to_blocks(
            self,
            markdown_content: str
    ) -> list[models.EntryVersionBlock]:
        syntax_tree = marko.parse(markdown_content)

        blocks = []
        for marko_block in syntax_tree.children:
            if isinstance(marko_block, List):
                blocks.extend(
                    self._convert_list_block(
                        marko_block=marko_block,
                        bullet=marko_block.bullet,
                        indent_level=-1,
                        ordered=marko_block.ordered,
                    )
                )
            else:
                blocks.append(
                    self._marko_block_to_version_block(
                        marko_block=marko_block,
                        order=self._order,
                    )
                )
                self._order += 1

        return blocks

    @staticmethod
    def _get_marko_text_content(marko_block: BlockElement) -> str:
        html_text = marko.render(marko_block)
        bolded_text = re.sub(r'<strong>(.*?)</strong>', r'**\1**', html_text)
        italicized_text = re.sub(r'<em>(.*?)</em>', r'*\1*', bolded_text)
        strikethrough_text = re.sub(r'<del>(.*?)</del>', r'~~*\1*~~', italicized_text)
        text = re.sub(r'<[^>]+>', '', strikethrough_text)
        return html.unescape(text)

    def _marko_block_to_version_block(
            self,
            marko_block: BlockElement | Element,
            order: int
    ) -> models.EntryVersionBlock:
        if isinstance(marko_block, Heading):
            return self._convert_heading_block(marko_block=marko_block, order=order)

        return models.EntryVersionBlock.services.factory.create_null_block(
            entry_version=self.entry_version,
            block_type=models.BlockTypeChoices.TEXT,
            order=order,
            value=self._get_marko_text_content(marko_block)
        )



from django.urls import path

from django_spire.knowledge.entry.version.views import json_views

app_name = 'json'

urlpatterns = [
    path(
        '<int:pk>/create_blank_block/',
        json_views.create_blank_block_view,
        name='create_blank_block',
    ),
    path(
        '<int:pk>/delete_block/',
        json_views.delete_block_view,
        name='delete_block',
    ),
    path(
        '<int:pk>/reorder/',
        json_views.reorder_view,
        name='reorder',
    ),
]



from __future__ import annotations

from django_spire.auth.group.models import AuthGroup


def bulk_create_groups_from_names(names: list[str]):
    existing_groups = AuthGroup.objects.all().values_list('name', flat=True)

    new_groups = [
        AuthGroup(name=name)
        for name in names
        if name not in existing_groups
    ]

    return AuthGroup.objects.bulk_create(new_groups)



from __future__ import annotations

from django.db import models
from django.urls import reverse

from django_spire.contrib.breadcrumb import Breadcrumbs
from django_spire.history.mixins import HistoryModelMixin

from module import querysets
from module.services.service import SpireChildAppService


class SpireChildApp(HistoryModelMixin):
    name = models.CharField(max_length=255)
    description = models.TextField(default='')

    objects = querysets.SpireChildAppQuerySet().as_manager()
    services = SpireChildAppService()

    def __str__(self):
        return self.name

    @classmethod
    def base_breadcrumb(cls) -> Breadcrumbs:
        crumbs = Breadcrumbs()

        crumbs.add_breadcrumb(
            'SpireChildApp',
            reverse('spireparentapp:spirechildapp:page:list')
        )

        return crumbs

    def breadcrumbs(self) -> Breadcrumbs:
        crumbs = Breadcrumbs()
        crumbs.add_base_breadcrumb(self._meta.model)

        if self.pk:
            crumbs.add_breadcrumb(
                str(self),
                reverse(
                    'spireparentapp:spirechildapp:page:detail',
                    kwargs={'pk': self.pk}
                )
            )

        return crumbs

    class Meta:
        verbose_name = 'SpireChildApp'
        verbose_name_plural = 'SpireChildApps'
        db_table = 'spireparentapp_spirechildapp'



from importlib import import_module

from dandy.llm import MessageHistory
from dandy.workflow import BaseWorkflow
from django.conf import settings
from django.contrib.auth.base_user import AbstractBaseUser
from django.core.handlers.wsgi import WSGIRequest

from django_spire.ai.decorators import log_ai_interaction_from_recorder
from django_spire.ai.sms.intel import SmsIntel
from django_spire.consts import AI_SMS_CONVERSATION_WORKFLOW_CLASS_SETTINGS_NAME


def sms_workflow_process(
        request: WSGIRequest,
        user_input: str | None = None,
        message_history: MessageHistory | None = None,
        user: AbstractBaseUser | None = None,
        actor: str | None = None,
) -> SmsIntel:

    if user_input is None:
        raise ValueError('sms user_input is required')

    sms_workflow_class = getattr(settings, AI_SMS_CONVERSATION_WORKFLOW_CLASS_SETTINGS_NAME)
    
    if sms_workflow_class is None:
        raise ValueError(f'"{AI_SMS_CONVERSATION_WORKFLOW_CLASS_SETTINGS_NAME}" must be set in the django settings.')

    module_name = '.'.join(sms_workflow_class.split('.')[:-1])
    object_name = sms_workflow_class.split('.')[-1]

    try:
        workflow_module = import_module(module_name)
    except ImportError:
        raise ImportError(f'Could not import workflow module: {module_name}')

    SmsWorkFlow: BaseWorkflow = getattr(workflow_module, object_name)

    @log_ai_interaction_from_recorder(user=user, actor=actor)
    def run_workflow_process() -> SmsIntel:
        return SmsWorkFlow.process(
            request=request,
            user_input=user_input,
            message_history=message_history,
        )

    output_intel = run_workflow_process()

    if not issubclass(output_intel.__class__, SmsIntel):
        raise ValueError(
            f'{SmsWorkFlow.__class__.__module__}.{SmsWorkFlow.__class__.__qualname__}.process must return an instance of a {SmsIntel.__name__}.'
        )

    return output_intel



from django.urls import path

from django_spire.auth.group.views import page_views


app_name = 'page'

# Group
urlpatterns = [
    path('group/<int:pk>/detail/',
         page_views.detail_view,
         name='detail'),

    path('group/list/',
         page_views.list_view,
         name='list'),
]



from django.urls import path

from django_spire.knowledge.entry.version.views import page_views

app_name = 'page'

urlpatterns = [
    path('<int:pk>/detail/', page_views.detail_view, name='detail'),
]



from django.urls.conf import path, include


app_name = 'theme'

urlpatterns = [
    path('ajax/', include('django_spire.theme.urls.ajax_urls', namespace='ajax')),
    path('page/', include('django_spire.theme.urls.page_urls', namespace='page')),
]



def get_object_from_module_string(module_string):
    try:
        module_string, object_name = module_string.rsplit('.', 1)
        module = __import__(module_string, fromlist=[object_name])
    except ImportError:
        raise ImportError(f'Could not import module: {module_string}')

    return getattr(module, object_name)


from django.urls import include, path

app_name = 'block'

urlpatterns = [
    path(
        'json/',
        include(
            'django_spire.knowledge.entry.version.block.urls.json_urls',
            namespace='json'
        )
    ),
]



from __future__ import annotations


def generate_replacement_map(components: list[str]) -> dict[str, str]:
    app_name = components[-1]
    class_name = ''.join(word.title() for word in app_name.split('_'))
    parent = components[-2] if len(components) > 1 else app_name
    module = '.'.join(components)

    return {
        'module': module,
        'spirepermission': parent.lower() + app_name.lower(),
        'SpireChildApp': class_name,
        'SpireChildApps': class_name + 's',
        'spirechildapp': app_name.lower(),
        'spirechildapps': app_name.lower() + 's',
        'SpireParentApp': class_name,
        'SpireParentApps': class_name + 's',
        'spireparentapp': parent.lower(),
        'spireparentapps': parent.lower() + 's'
    }



from __future__ import annotations

from pathlib import Path

from django.test import TestCase

from django_spire.theme.enums import ThemeFamily, ThemeMode
from django_spire.theme.models import Theme


class ThemeFilesystemValidationTests(TestCase):
    def setUp(self) -> None:
        self.base_path = self._get_themes_base_path()

    def _get_themes_base_path(self) -> Path:
        file = Path(__file__)
        root = file.parent.parent.parent.parent
        return root / 'django_spire' / 'static' / 'django_spire' / 'css' / 'themes'

    def test_themes_directory_exists(self) -> None:
        if not self.base_path.exists():
            self.skipTest(f'Themes directory does not exist: {self.base_path}. Create it to enable filesystem validation tests.')

        self.assertTrue(self.base_path.is_dir(), f'Themes path is not a directory: {self.base_path}')

    def test_all_theme_families_have_directories(self) -> None:
        if not self.base_path.exists():
            self.skipTest(f'Themes directory does not exist: {self.base_path}')

        existing = {path.name for path in self.base_path.iterdir() if path.is_dir()}
        expected = {family.value for family in ThemeFamily}

        missing = expected - existing
        if missing:
            self.fail(f'Missing theme directories: {sorted(missing)}. Create these directories: {sorted(missing)}')

    def test_no_unexpected_theme_directories(self) -> None:
        if not self.base_path.exists():
            self.skipTest(f'Themes directory does not exist: {self.base_path}')

        existing = {path.name for path in self.base_path.iterdir() if path.is_dir()}
        expected = {family.value for family in ThemeFamily}

        unexpected = existing - expected

        if unexpected:
            self.fail(f'Unexpected theme directories: {sorted(unexpected)}. Remove these or update ThemeFamily enum: {sorted(unexpected)}')

    def test_all_theme_mode_files_exist(self) -> None:
        if not self.base_path.exists():
            self.skipTest(f'Themes directory does not exist: {self.base_path}')

        missing = []

        for family in ThemeFamily:
            directory = self.base_path / family.value

            if not directory.exists():
                missing.extend([f'{family.value}/app-{mode.value}.css' for mode in ThemeMode])
                continue

            for mode in ThemeMode:
                file = directory / f'app-{mode.value}.css'
                if not file.exists():
                    missing.append(f'{family.value}/app-{mode.value}.css')

        if missing:
            self.fail(f'Missing CSS files: {sorted(missing)}')

    def test_no_unexpected_css_files(self) -> None:
        if not self.base_path.exists():
            self.skipTest(f'Themes directory does not exist: {self.base_path}')

        expected = {f'app-{mode.value}.css' for mode in ThemeMode}
        unexpected = {}

        for family in ThemeFamily:
            directory = self.base_path / family.value

            if directory.exists():
                existing = {f.name for f in directory.iterdir() if f.is_file() and f.suffix == '.css'}
                files = existing - expected

                if files:
                    unexpected[family.value] = sorted(files)

        if unexpected:
            self.fail(f'Unexpected CSS files by family: {unexpected}')

    def test_theme_stylesheet_paths_match_filesystem(self) -> None:
        if not self.base_path.exists():
            self.skipTest(f'Themes directory does not exist: {self.base_path}')

        missing = []

        for theme in Theme.get_available():
            expected = theme.stylesheet

            file = Path(__file__)
            root = file.parent.parent.parent.parent
            path = root / 'django_spire' / 'static' / expected

            if not path.exists():
                missing.append(f'{theme.value}: {expected}')

        if missing:
            self.fail(f'Missing stylesheet files: {missing}')

    def test_css_files_are_not_empty(self) -> None:
        if not self.base_path.exists():
            self.skipTest(f'Themes directory does not exist: {self.base_path}')

        empty = []

        for family in ThemeFamily:
            directory = self.base_path / family.value

            if directory.exists():
                for mode in ThemeMode:
                    file = directory / f'app-{mode.value}.css'

                    if file.exists():
                        size = file.stat().st_size
                        if size == 0:
                            empty.append(f'{family.value}/app-{mode.value}.css')

        if empty:
            self.fail(f'Empty CSS files: {sorted(empty)}')

    def test_filesystem_structure_completeness(self) -> None:
        if not self.base_path.exists():
            self.skipTest(f'Themes directory does not exist: {self.base_path}')

        total_expected_families = len(ThemeFamily)
        total_expected_modes = len(ThemeMode)
        total_expected_files = total_expected_families * total_expected_modes

        actual = 0

        for family in ThemeFamily:
            directory = self.base_path / family.value

            if directory.exists():
                files = list(directory.glob('app-*.css'))
                actual += len(files)

        if actual != total_expected_files:
            self.fail(f'Expected {total_expected_files} CSS files, found {actual}')

    def test_directory_structure_matches_enum_values(self) -> None:
        if not self.base_path.exists():
            self.skipTest(f'Themes directory does not exist: {self.base_path}')

        missing = []

        for family in ThemeFamily:
            directory = self.base_path / family.value

            if not directory.exists():
                missing.append(family.value)

        if missing:
            self.fail(f'Missing directories for theme families: {sorted(missing)}')

    def test_css_filenames_match_enum_values(self) -> None:
        if not self.base_path.exists():
            self.skipTest(f'Themes directory does not exist: {self.base_path}')

        for family in ThemeFamily:
            directory = self.base_path / family.value

            if directory.exists():
                for mode in ThemeMode:
                    filename = f'app-{mode.value}.css'
                    file = directory / filename

                    if file.exists():
                        self.assertEqual(
                            file.name, filename,
                            f'Filename {file.name} does not match expected {filename}'
                        )



from __future__ import annotations

from typing_extensions import TYPE_CHECKING

from django.contrib.auth.decorators import permission_required
from django.shortcuts import get_object_or_404
from django.template.response import TemplateResponse

from django_spire.contrib.generic_views import portal_views

from module import forms, models


if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest


@permission_required('spireparentapp.view_spirepermission')
def detail_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    spirechildapp = get_object_or_404(models.SpireChildApp, pk=pk)

    context_data = {
        'spirechildapp': spirechildapp,
    }

    return portal_views.detail_view(
        request,
        obj=spirechildapp,
        context_data=context_data,
        template='spirechildapp/page/detail_page.html'
    )


@permission_required('spireparentapp.view_spirepermission')
def list_view(request: WSGIRequest) -> TemplateResponse:
    context_data = {
        'spirechildapps': models.SpireChildApp.objects.all()
    }

    return portal_views.list_view(
        request,
        model=models.SpireChildApp,
        context_data=context_data,
        template='spirechildapp/page/list_page.html'
    )



from dandy.llm import BaseLlmBot, LlmConfigOptions, Prompt

from django_spire.ai.prompt.tuning import prompts, intel



class PromptTestingBot(BaseLlmBot):
    instructions_prompt = Prompt()
    intel_class = intel.PromptTestingIntel
    config_options = LlmConfigOptions(temperature=0.4)

    @classmethod
    def process(
            cls,
            system_prompt: str,
            user_prompt: str,

    ) -> intel.PromptTuningIntel:
        """
            Takes a system prompt and a user prompt.
            Used to test the response of the system prompt.
        """

        cls.instructions_prompt = system_prompt
        return cls.process_prompt_to_intel(
            prompt=user_prompt
        )


class SimplePromptTuningBot(BaseLlmBot):
    instructions_prompt = prompts.prompt_tuning_instruction_bot_prompt()
    intel_class = intel.PromptTuningIntel
    config_options = LlmConfigOptions(temperature=0.1)


    @classmethod
    def process(
            cls,
            prompt: str,
            feedback: str,

    ) -> intel.PromptTuningIntel:

        return cls.process_prompt_to_intel(
            prompt=prompts.prompt_tuning_input_prompt(prompt, feedback),
        )


class AdvancedPromptTuningBot(BaseLlmBot):
    instructions_prompt = prompts.prompt_tuning_instruction_bot_prompt()
    intel_class = intel.PromptTuningIntel

    @classmethod
    def process(
            cls,
            system_prompt: str,
            feedback: str,

    ) -> intel.PromptTuningIntel:
        tuned_prompt = SimplePromptTuningBot.process(system_prompt, feedback)
        formatted_prompt = FormattingBot.process(tuned_prompt.prompt)
        remove_duplicates = DuplicationRemovalBot.process(formatted_prompt.prompt)
        improve_instructions = InstructionClarityBot.process(remove_duplicates.prompt)
        example_optimization = ExampleOptimizationBot.process(improve_instructions.prompt)
        persona = PersonaBot.process(example_optimization.prompt)
        return persona


        # return cls.process_prompt_to_intel(
        #     prompt=prompts.prompt_tuning_input_prompt(system_prompt, feedback),
        #     postfix_system_prompt=None
        # )
        #

class FormattingBot(BaseLlmBot):
    """Bot that preserves structure and standardizes formatting of system prompts."""
    instructions_prompt = prompts.formatting_bot_instruction_prompt()
    intel_class = intel.PromptTuningIntel

    @classmethod
    def process(
            cls,
            system_prompt: str,

    ) -> intel.PromptTuningIntel:

        return cls.process_prompt_to_intel(
            prompt=prompts.specialized_bot_input_prompt(system_prompt),
            postfix_system_prompt=None
        )


class InstructionClarityBot(BaseLlmBot):
    """Bot that focuses on improving the clarity of instructions in system prompts."""
    instructions_prompt = prompts.instruction_clarity_bot_instruction_prompt()
    intel_class = intel.PromptTuningIntel

    @classmethod
    def process(
            cls,
            system_prompt: str,

    ) -> intel.PromptTuningIntel:

        return cls.process_prompt_to_intel(
            prompt=prompts.specialized_bot_input_prompt(system_prompt),
            postfix_system_prompt=None
        )


class PersonaBot(BaseLlmBot):
    """Bot that maintains consistent tone and persona throughout system prompts."""
    instructions_prompt = prompts.persona_bot_instruction_prompt()
    intel_class = intel.PromptTuningIntel

    @classmethod
    def process(
            cls,
            system_prompt: str,

    ) -> intel.PromptTuningIntel:

        return cls.process_prompt_to_intel(
            prompt=prompts.specialized_bot_input_prompt(system_prompt),
            postfix_system_prompt=None
        )


class DuplicationRemovalBot(BaseLlmBot):
    """Bot that identifies and removes redundancies in system prompts."""
    instructions_prompt = prompts.duplication_removal_bot_instruction_prompt()
    intel_class = intel.PromptTuningIntel

    @classmethod
    def process(
            cls,
            system_prompt: str,

    ) -> intel.PromptTuningIntel:

        return cls.process_prompt_to_intel(
            prompt=prompts.specialized_bot_input_prompt(system_prompt),
            postfix_system_prompt=None
        )


class ExampleOptimizationBot(BaseLlmBot):
    """Bot that refines examples within system prompts."""
    instructions_prompt = prompts.example_optimization_bot_instruction_prompt()
    intel_class = intel.PromptTuningIntel

    @classmethod
    def process(
            cls,
            system_prompt: str,

    ) -> intel.PromptTuningIntel:

        return cls.process_prompt_to_intel(
            prompt=prompts.specialized_bot_input_prompt(system_prompt),
            postfix_system_prompt=None
        )



from __future__ import annotations

from typing import TYPE_CHECKING

from django.db.models import Count

from django_spire.contrib.ordering.querysets import OrderingQuerySetMixin
from django_spire.history.querysets import HistoryQuerySet

if TYPE_CHECKING:
    from django.db.models import QuerySet
    from django_spire.knowledge.collection.models import Collection


class CollectionQuerySet(HistoryQuerySet, OrderingQuerySetMixin):
    def annotate_entry_count(self) -> QuerySet[Collection]:
        return self.annotate(entry_count=Count('entry'))

    def by_parent(self, parent: Collection) -> QuerySet[Collection]:
        return self.filter(parent=parent)

    def by_parent_id(self, parent_id: int) -> QuerySet[Collection]:
        return self.filter(parent_id=parent_id)

    def childless(self) -> QuerySet[Collection]:
        return self.annotate(child_count=Count('child')).filter(child_count=0)

    def parentless(self) -> QuerySet[Collection]:
        return self.filter(parent_id__isnull=True)



from django.db import connections
from functools import wraps

from django.http import JsonResponse


def close_db_connections(func):
    @wraps(func)
    def inner(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        finally:
            connections.close_all()
    return inner


def valid_ajax_request_required(method):
    @wraps(method)
    def wrapper(request, *args, **kwargs):
        if request.method != 'POST' and request.content_type != 'application/json':
            return JsonResponse(
                {'type': 'error', 'message': 'Invalid Request'}
            )
        return method(request, *args, **kwargs)
    return wrapper



from django_spire.knowledge.entry.version.block.choices import BlockTypeChoices
from django_spire.knowledge.entry.version.block.blocks.block import BaseBlock


class SubHeadingBlock(BaseBlock):
    value: str
    type: BlockTypeChoices = BlockTypeChoices.SUB_HEADING
    detail_template: str = 'django_spire/knowledge/entry/version/block/detail/component/sub_heading_component.html'
    update_template: str = 'django_spire/knowledge/entry/version/block/update/component/sub_heading_component.html'

    def render_to_text(self) -> str:
        return f'{self.value}\n'



from typing import Type

from dandy.llm import BaseLlmMap
from dandy.map import Map

from django_spire.knowledge.collection.models import Collection


def get_collection_map_class() -> Type[BaseLlmMap]:
    class CollectionMap(BaseLlmMap):
        map_keys_description = 'Knowledge Collection Titles'
        map = Map({
            **{
                collection.name: collection
                for collection in Collection.objects.all().annotate_entry_count()
            },
            **{'No Matching Knowledge Collection Titles': None}
        })

    return CollectionMap


from __future__ import annotations

import django_spire

from pathlib import Path
from typing_extensions import TYPE_CHECKING

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError

from django_spire.core.management.commands.spire_startapp_pkg.manager import (
        AppManager,
        HTMLTemplateManager,
)
from django_spire.core.management.commands.spire_startapp_pkg.processor import (
        AppTemplateProcessor,
        HTMLTemplateProcessor
)
from django_spire.core.management.commands.spire_startapp_pkg.reporter import Reporter

if TYPE_CHECKING:
    import argparse


class Command(BaseCommand):
    help = 'Create a custom Spire app.'

    def __init__(self):
        super().__init__()

        self.app_base = Path(settings.BASE_DIR)
        self.app_template = Path(django_spire.__file__).parent / 'core/management/commands/spire_startapp_pkg/template/app'

        self.template_base = self.app_base / 'templates'
        self.html_template = Path(django_spire.__file__).parent / 'template/templates'

        self.app_manager = AppManager(self.app_base, self.app_template)
        self.app_processor = AppTemplateProcessor()

        self.html_manager = HTMLTemplateManager(self.template_base, self.html_template)
        self.html_processor = HTMLTemplateProcessor()

        self.reporter = Reporter(self)

    def get_app_names(self) -> list[str]:
        from django.apps import apps
        return [config.name for config in apps.get_app_configs()]

    def handle(self, *_args, **kwargs) -> None:
        app = input('Enter the path of the app (e.g., "app.maintenance.work_order"):')

        if not app:
            raise CommandError(self.style.ERROR('The app name is missing'))

        self.app_manager.validate_app_name_format(app)

        components = self.app_manager.parse_app_name(app)
        self.app_manager.is_valid_root_apps(components)

        registry = self.get_app_names()

        self.reporter.write(
            f'Checking app components: {components}\n\n',
            self.style.NOTICE
        )

        missing = self.app_manager.get_missing_components(components, registry)

        if missing:
            self.reporter.report_missing_components(missing)
            self.reporter.report_app_tree_structure(self.app_base, components, registry, self.app_template)
            # self.reporter.report_html_tree_structure(self.template_base, components, registry, self.html_template)

            if not self.reporter.prompt_for_confirmation('\nProceed with app creation? (y/n): '):
                self.reporter.write('App creation aborted.', self.style.ERROR)
                return

            for module in [missing[-1]]:
                self.app_manager.create_custom_app(module, self.app_processor, self.reporter)
                # self.html_manager.create_custom_templates(module, self.html_processor, self.reporter)

            self.reporter.report_installed_apps_suggestion(missing)
        else:
            self.reporter.write('All component(s) exist.', self.style.SUCCESS)



from django.apps import AppConfig
from django.conf import settings

from django_spire.consts import NOTIFICATION_THROTTLE_RATE_PER_MINUTE_SETTINGS_NAME
from django_spire.utils import check_required_apps


class NotificationConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'django_spire.notification'
    label = 'django_spire_notification'

    REQUIRED_APPS = ('django_spire_core', 'django_spire_history', 'django_spire_history_viewed')

    URLPATTERNS_INCLUDE = 'django_spire.notification.urls'
    URLPATTERNS_NAMESPACE = 'notification'

    def ready(self) -> None:
        if not isinstance(getattr(settings, NOTIFICATION_THROTTLE_RATE_PER_MINUTE_SETTINGS_NAME), int):
            raise ValueError(f'"{NOTIFICATION_THROTTLE_RATE_PER_MINUTE_SETTINGS_NAME}" must be set in the django settings when using "{self.label}".')

        check_required_apps(self.label)



from django_spire.history.querysets import HistoryQuerySet
from django_spire.notification.querysets import NotificationContentObjectQuerySet


class SmsNotificationQuerySet(HistoryQuerySet, NotificationContentObjectQuerySet):
    pass


class SmsTemporaryMediaQuerySet(HistoryQuerySet):
    def is_ready_for_deletion(self):
        return [media for media in self if media.is_ready_for_deletion()]



from django.db import models

from django_spire.contrib.ordering.mixins import OrderingModelMixin
from django_spire.history.mixins import HistoryModelMixin
from django_spire.knowledge.collection.models import Collection
from django_spire.knowledge.entry.querysets import EntryQuerySet
from django_spire.knowledge.entry.services.service import EntryService
from django_spire.knowledge.entry.version.models import EntryVersion


class Entry(HistoryModelMixin, OrderingModelMixin):
    collection = models.ForeignKey(
        Collection,
        on_delete=models.CASCADE,
        related_name='entries',
        related_query_name='entry'
    )
    current_version = models.OneToOneField(
        EntryVersion,
        on_delete=models.CASCADE,
        related_name='current_version',
        related_query_name='current_version',
        null=True,
        blank=True,
    )

    name = models.CharField(max_length=255)

    objects = EntryQuerySet.as_manager()
    services = EntryService()

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = 'Entry'
        verbose_name_plural = 'Entries'
        db_table = 'django_spire_knowledge_entry'



import logging
import time

from django.conf import settings


def performance_timer(func):
    def wrapper(*args, **kwargs):
        if settings.DEBUG:
            start_time = time.perf_counter()
            result = func(*args, **kwargs)
            end_time = time.perf_counter()

            logging.warning(f'{end_time - start_time:.4f} seconds runtime for "{func.__module__}.{func.__qualname__}"')

            return result

        else:
            return func(*args, **kwargs)

    return wrapper


from __future__ import annotations

from typing_extensions import TYPE_CHECKING

from django_spire.history.querysets import HistoryQuerySet

if TYPE_CHECKING:
    from django.db.models import QuerySet


class SpireChildAppQuerySet(HistoryQuerySet):
    pass



from django.urls import path

from module.views import form_views


app_name = 'form'

urlpatterns = [
    path('<int:pk>/delete/form/', form_views.delete_view, name='delete'),
    path('<int:pk>/delete/form/modal/', form_views.delete_form_modal_view, name='delete_form_modal'),
    path('<int:pk>/form/content/', form_views.form_content_modal_view, name='form_modal_content'),
    path('<int:pk>/form/', form_views.form_view, name='form'),
]



from __future__ import annotations

from django.contrib import messages
from django.http import HttpResponseRedirect
from django.template.response import TemplateResponse
from django.urls import reverse

from django_spire.auth.mfa.utils import get_or_generate_user_mfa_code
from django_spire.auth.mfa import forms


def mfa_form_view(request):
    mfa_code = get_or_generate_user_mfa_code(request.user)
    profile = request.user.profile

    if request.method == 'POST':
        form = forms.MFAForm(mfa_code, request.POST)
        if form.is_valid():
            mfa_code.set_expired()
            profile.set_mfa_grace_period()
            return HttpResponseRedirect(reverse('home:home'))

        if 'mfa_code' in form.errors:
            messages.error(request, form.errors['mfa_code'][0])

    context_data = {
        'form': forms.MFAForm(mfa_code, initial=request.POST),
        'form_description': 'An authentication code has been sent to your email. Please enter it below to continue.',
    }

    return TemplateResponse(
        request,
        context=context_data,
        template='django_spire/auth/mfa/mfa_form.html'
    )




# These imports are for the migrations to work as a single app "django_spire_knowledge"

from django_spire.knowledge.entry import models
from django_spire.knowledge.entry.version import models
from django_spire.knowledge.entry.version.block import models
from django_spire.knowledge.collection import models



from django.urls import path, include

from django_spire.ai.chat.views import message_views

app_name = 'message'

urlpatterns = [
    path('request/', include('django_spire.ai.chat.urls.message_request_urls', namespace='request')),
    path('response/', include('django_spire.ai.chat.urls.message_response_urls', namespace='response')),
]

urlpatterns += [
    path('load/chat/<int:chat_id>/', message_views.load_messages_render_view, name='load_chat'),
]




from dandy.llm import BaseLlmBot, Prompt


class MarkdownFormatLlmBot(BaseLlmBot):

    @classmethod
    def process(
            cls,
            markdown_content: str
    ) -> str:
        markdown_prompt = Prompt()
        markdown_prompt.text(
            'Can you improve the markdown formatting? Do NOT add or change any of the '
            'content.'
        )
        markdown_prompt.line_break()
        markdown_prompt.text(markdown_content)

        return cls.process_prompt_to_intel(prompt=markdown_prompt).text



from django import template
import json

register = template.Library()

@register.filter(name='json_loads')
def json_loads(value):
    try:
        return json.loads(value)
    except (TypeError, ValueError) as e:
        return {}


# Generated by Django 5.2.1 on 2025-06-04 23:48

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='SmsConversation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField(default=True, editable=False)),
                ('is_deleted', models.BooleanField(default=False, editable=False)),
                ('created_datetime', models.DateTimeField(default=django.utils.timezone.localtime, editable=False)),
                ('phone_number', models.CharField(max_length=20)),
                ('last_message_datetime', models.DateTimeField(default=django.utils.timezone.now, editable=False)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='sms_conversations', related_query_name='sms_conversation', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'SMS Conversation',
                'verbose_name_plural': 'SMS Conversations',
                'db_table': 'django_spire_ai_sms_conversation',
                'ordering': ('-last_message_datetime',),
            },
        ),
        migrations.CreateModel(
            name='SmsMessage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField(default=True, editable=False)),
                ('is_deleted', models.BooleanField(default=False, editable=False)),
                ('created_datetime', models.DateTimeField(default=django.utils.timezone.localtime, editable=False)),
                ('body', models.TextField()),
                ('is_inbound', models.BooleanField(default=False)),
                ('twilio_sid', models.CharField(blank=True, max_length=64, null=True)),
                ('is_processed', models.BooleanField(default=False)),
                ('conversation', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='messages', related_query_name='message', to='django_spire_ai_sms.smsconversation')),
            ],
            options={
                'verbose_name': 'SMS Message',
                'verbose_name_plural': 'SMS Messages',
                'db_table': 'django_spire_ai_sms_message',
                'ordering': ('-created_datetime',),
            },
        ),
    ]



from pathlib import Path

from dandy.llm import BaseLlmBot
from django.conf import settings

from django_spire.ai.prompt import prompts
from django_spire.ai.prompt import intel



class DandyPythonPromptBot(BaseLlmBot):
    instructions_prompt = prompts.dandy_prompt_python_file_instruction_bot_prompt()
    intel_class = intel.DandyPromptPythonFileIntel

    @classmethod
    def process(
            cls,
            prompt: str,

    ) -> intel.DandyPromptPythonFileIntel:
        """
            Takes in an enhanced prompt and turn it into a python file that uses the dandy library.
        """
        return cls.process_prompt_to_intel(
            prompt=prompts.dandy_prompt_python_file_input_prompt(prompt)
        )


class TextToMarkdownPromptBot(BaseLlmBot):
    instructions_prompt = prompts.text_to_markdown_instruction_bot_prompt()
    intel_class = intel.TextToMarkdownIntel

    @classmethod
    def process(
            cls,
            text: str,

    ) -> intel.TextToMarkdownIntel:
        """
            Takes in user text and converts it to properly formatted Markdown.
        """
        return cls.process_prompt_to_intel(
            prompt=prompts.text_to_markdown_input_prompt(text)
        )



from __future__ import annotations

from django.forms import Form
from typing_extensions import TYPE_CHECKING

from django.contrib import messages

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest


def show_form_errors(request: WSGIRequest, *forms: Form) -> None:
    for form in forms:
        for field_name, error_list in form.errors.items():
            for error in error_list.data:
                error_message = ''

                if field_name != '__all__':
                    error_message += f'{field_name.title()}: '

                if hasattr(error, 'message_responses'):
                    error_message += f'{" ".join(error.message_responses)}'

                elif hasattr(error, 'messages'):
                    error_message += f'{" ".join(error.messages)}'

                else:
                    raise Exception('Error message not found.')

                messages.error(request, error_message)



from __future__ import annotations

import json
from typing import TYPE_CHECKING

from django.contrib.contenttypes.models import ContentType

from django_spire.contrib.service import BaseDjangoModelService
from django_spire.file.models import File
from django_spire.knowledge.entry.tests.constants import ENTRY_IMPORT_RELATED_FIELD

if TYPE_CHECKING:
    from django.db.models import QuerySet
    from django_spire.knowledge.entry.models import Entry


class EntryToolService(BaseDjangoModelService['Entry']):
    obj: Entry

    def get_files_to_convert(self) -> QuerySet[File]:
        return (
            File.objects.related_field(field_name=ENTRY_IMPORT_RELATED_FIELD)
            .filter(content_type=ContentType.objects.get_for_model(self.obj_class))
            .active()
            .order_by('object_id')
        )

    def get_files_to_convert_json(self) -> str:
        return json.dumps(
            [
                {
                    'id': file_object.id,
                    'name': file_object.name,
                    'type': file_object.type,
                    'size': file_object.size,
                }
                for file_object in self.get_files_to_convert()
            ]
        )



from django.contrib import admin

from django_spire.notification.app.models import AppNotification


@admin.register(AppNotification)
class AppNotificationAdmin(admin.ModelAdmin):
    list_display = (
        'id', 'notification', 'created_datetime', 'is_deleted'
    )



from django.urls import path

from django_spire.theme.views import ajax_views

app_name = 'ajax'

urlpatterns = [
    path('get_config/', ajax_views.get_config, name='get_config'),
    path('set_theme/', ajax_views.set_theme, name='set_theme'),
]



from __future__ import annotations
from typing import TYPE_CHECKING

from django_spire.contrib.service import BaseDjangoModelService
from module.services.factory_service import SpireChildAppFactoryService
from module.services.processor_service import SpireChildAppProcessorService
from module.services.intelligence_service import SpireChildAppIntelligenceService
from module.services.transformation_service import SpireChildAppTransformationService

if TYPE_CHECKING:
    from module.models import SpireChildApp


class SpireChildAppService(BaseDjangoModelService['SpireChildApp']):
    obj: SpireChildApp

    intelligence = SpireChildAppIntelligenceService()
    processor = SpireChildAppProcessorService()
    factory = SpireChildAppFactoryService()
    transformation = SpireChildAppTransformationService()



# from django_spire.seeding.intelligence.bots import LlmSeedingBot
# from django_spire.seeding.seeder import BaseSeeder
#
# from dandy.llm import Prompt
# from dandy.intel import BaseIntel
#
#
# class LlmSeeder(BaseSeeder):
#     keyword = 'llm'
#
#     def __init__(
#             self,
#             fields: dict = None,
#             default_to: str = "llm",
#             prompt: Prompt = None,
#      ):
#         super().__init__(fields, default_to)
#         self.prompt = prompt or Prompt()
#
#
#     def seed(self, manager, count = 1) -> list[dict]:
#         # Todo: Need to turn fields into llm call?
#         # How does that work with pydantic?
#
#         class SeedingIntel(BaseIntel):
#             pass
#         #     items: list[seed_intel_class]
#         #
#         #     def __iter__(self):
#         #         return iter(self.items)
#
#         prompt = (
#             Prompt()
#             .prompt(self.prompt)
#             .heading('Seed Count')
#             # .text(f'Create {count} {self.model_class.__name__}')
#         )
#
#         intel_data = LlmSeedingBot.process(
#             prompt=prompt,
#             intel_class=SeedingIntel
#         )
#
#         return intel_data



from __future__ import annotations

from typing import TYPE_CHECKING

from django_spire.contrib.service import BaseDjangoModelService

if TYPE_CHECKING:
    from django_spire.knowledge.collection.models import Collection


class CollectionProcessorService(BaseDjangoModelService['Collection']):
    obj: Collection

    def set_deleted(self):
        if self.obj.parent_id is None:
            destination_objects = self.obj_class.objects.parentless().active()
        else:
            destination_objects = (
                self.obj_class.objects
                .by_parent_id(parent_id=self.obj.parent_id)
                .active()
            )

        self.obj.ordering_services.processor.remove_from_objects(
            destination_objects=destination_objects
        )
        self.obj.set_deleted()

        for child in self.obj.children.active():
            child.services.processor.set_deleted()



# Generated by Django 5.2.1 on 2025-06-03 19:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_spire_notification', '0002_pushnotification'),
    ]

    operations = [
        migrations.DeleteModel(
            name='PushNotification',
        ),
    ]



from __future__ import annotations

from typing import TYPE_CHECKING, Type

from dandy.map import Map
from dandy.recorder import recorder_to_html_file
from dandy.workflow import BaseWorkflow
from dandy.llm import LlmBot, MessageHistory, Prompt
from django.core.handlers.wsgi import WSGIRequest

from django_spire.ai.chat.intelligence.maps.intent_llm_map import IntentLlmMap
from django_spire.ai.chat.intelligence.prompts import organization_prompt
from django_spire.ai.chat.message_intel import DefaultMessageIntel
from django_spire.auth.controller.controller import AppAuthController
from django_spire.knowledge.intelligence.workflows.knowledge_workflow import \
    KnowledgeWorkflow


if TYPE_CHECKING:
    from django_spire.ai.chat.message_intel import BaseMessageIntel


class SpireChatWorkflow(BaseWorkflow):
    @classmethod
    def _generate_intent_map(cls, request: WSGIRequest) -> Type[IntentLlmMap]:
        intent_dict = {}

        if AppAuthController(app_name='knowledge', request=request).can_view():
            intent_message = (
                'The user is looking for information or knowledge on something.'
            )
            intent_dict[intent_message] = KnowledgeWorkflow

        intent_dict['None of the above choices match the user\'s intent'] = None

        IntentLlmMap.map = Map(intent_dict)
        IntentLlmMap._map_enum = IntentLlmMap.map.as_enum()
        return IntentLlmMap

    @classmethod
    @recorder_to_html_file('spire_chat_workflow')
    def process(
            cls,
            request: WSGIRequest,
            user_input: str,
            message_history: MessageHistory | None = None
    ) -> BaseMessageIntel:
        intent_map = cls._generate_intent_map(request)
        intents = intent_map.process(user_input, max_return_values=1)

        if intents[0] is None:
            response = LlmBot.process(
                prompt=user_input,
                message_history=message_history,
                postfix_system_prompt=organization_prompt()
            )

            return DefaultMessageIntel(text=response.text)

        response = LlmBot.process(
            prompt=(
                Prompt()
                .text(f'User Input: {user_input}')
                .line_break()
                .text(
                    'Use the following information to the answer the user\'s question '
                    'or concern.'
                )
                .line_break()
                .text(intents[0].process(user_input))
            ),
            message_history=message_history,
            postfix_system_prompt=organization_prompt()
        )

        return DefaultMessageIntel(text=response.text)



from dandy.llm import Prompt


def spirechildapp_instruction_prompt():
    return (
        Prompt()
        .title('SpireChildApp Operations')
        .line_break()
        .heading('Purpose')
        .text('This prompt provides instructions for SpireChildApp operations.')
        .line_break()
        .heading('Instructions')
        .ordered_list([
            'Instruction 1',
            'Instruction 2',
            'Instruction 3',
        ])
        .line_break()
    )


def spirechildapp_user_input_prompt(user_input: str):
    return (
        Prompt()
        .heading('SpireChildApp Request')
        .text('The user wants to perform the following operation:')
        .text('')
        .text(user_input)
        .text('')
    )



from django.urls import path

from django_spire.auth.views import admin_views


app_name = 'admin'

urlpatterns = [
    path('login/',
         admin_views.LoginView.as_view(),
         name='login'),
]


# Password Views
urlpatterns += [
    path('password/change/',
         admin_views.PasswordChangeView.as_view(),
         name='password_change'),

    path('password/change/done/',
         admin_views.PasswordChangeDone.as_view(),
         name='password_change_done'),

    path('password/reset/',
         admin_views.PasswordResetView.as_view(),
         name='password_reset'),

    path('password/reset/complete/',
         admin_views.PasswordResetComplete.as_view(),
         name='password_reset_complete'),

    path('password/reset/confirmation/<uidb64>/<token>/',
         admin_views.PasswordResetConfirmation.as_view(),
         name='password_reset_confirmation'),

    path('password/reset/done/',
         admin_views.PasswordResetDone.as_view(),
         name='password_reset_done'),

    path('password/reset/key/form/',
         admin_views.PasswordResetKeyForm.as_view(),
         name='password_reset_key_form'),

    path('password/reset/key/form/done/',
         admin_views.PasswordResetKeyFormDone.as_view(),
         name='password_reset_key_form_done'),

    path('password/reset/set/',
         admin_views.PasswordSetForm.as_view(),
         name='password_set_form'),
]



from django.apps import AppConfig
from django.conf import settings

from django_spire.consts import AI_CHAT_WORKFLOW_CLASS_SETTINGS_NAME
from django_spire.utils import check_required_apps


class AiChatConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'django_spire.ai.chat'
    label = 'django_spire_ai_chat'
    MODEL_PERMISSIONS = (
        {
            'name': 'ai_chat',
            'verbose_name': 'AI Chat',
            'model_class_path': 'django_spire.ai.chat.models.Chat',
            'is_proxy_model': False,
        },
    )

    REQUIRED_APPS = ('django_spire_ai',)

    def ready(self) -> None:
        if not isinstance(getattr(settings, AI_CHAT_WORKFLOW_CLASS_SETTINGS_NAME), str):
            raise ValueError(f'"{AI_CHAT_WORKFLOW_CLASS_SETTINGS_NAME}" must be set in the django settings when using "{self.label}".')

        check_required_apps(self.label)



from __future__ import annotations

from typing import TYPE_CHECKING

from abc import ABC, abstractmethod
from typing import Any

from django.template.loader import render_to_string
from pydantic import BaseModel

from django_spire.knowledge.entry.version.block.choices import BlockTypeChoices

if TYPE_CHECKING:
    from django_spire.knowledge.entry.version.block.models import EntryVersionBlock


class BaseBlock(ABC, BaseModel):
    value: Any
    type: BlockTypeChoices
    detail_template: str
    update_template: str

    def __init_subclass__(cls, *args, **kwargs):
        super().__init_subclass__(*args, **kwargs)

        if cls.detail_template is None or cls.detail_template == '':
            raise ValueError(
                f'{cls.__module__}.{cls.__qualname__}.detail_template must be set'
            )

        if cls.update_template is None or cls.update_template == '':
            raise ValueError(
                f'{cls.__module__}.{cls.__qualname__}.update_template must be set'
            )

    @abstractmethod
    def render_to_text(self) -> str:
        raise NotImplementedError

    def to_dict(self, version_block: EntryVersionBlock):
        return {
            'value': self.value,
            'type': self.type,
            'update_template_rendered': render_to_string(
                context={
                    'version_block': version_block,
                    'value': self.value,
                },
                template_name=self.update_template,
            )
        }



from django.http import HttpResponse, HttpResponseForbidden
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from twilio.twiml.messaging_response import MessagingResponse

from django_spire.ai.sms.decorators import twilio_auth_required
from django_spire.ai.sms.models import SmsConversation
from django_spire.ai.sms.tools import sms_workflow_process


@csrf_exempt
@require_POST
@twilio_auth_required
def webhook_view(request):
    from_number = request.POST.get('From', '')

    if len(from_number) < 11:
        return HttpResponseForbidden()

    body = request.POST.get('Body', '')
    message_sid = request.POST.get('MessageSid', '')

    conversation, created = SmsConversation.objects.get_or_create(
        phone_number=from_number
    )

    message = conversation.add_message(
        body=body,
        is_inbound=True,
        twilio_sid=message_sid,
    )

    try:

        sms_intel = sms_workflow_process(
            request=request,
            user_input=body,
            message_history=conversation.generate_message_history(),
            actor=from_number,
        )

        twiml_response = MessagingResponse()
        twiml_response.message(sms_intel.body)

        conversation.add_message(
            body=sms_intel.body,
            is_inbound=False,
            twilio_sid=message_sid,
            is_processed=True
        )

        message.is_processed = True
        message.save()

        return HttpResponse(twiml_response)

    except:
        raise






import json

from django.core.handlers.wsgi import WSGIRequest
from django.shortcuts import get_object_or_404
from django.template.response import TemplateResponse
from django.urls import reverse

from django_spire.auth.controller.controller import AppAuthController
from django_spire.contrib import Breadcrumbs
from django_spire.contrib.generic_views import portal_views
from django_spire.knowledge.entry.version.block.choices import BlockTypeChoices
from django_spire.knowledge.entry.version.block.models import EntryVersionBlock
from django_spire.knowledge.entry.version.models import EntryVersion


@AppAuthController('knowledge').permission_required('can_change')
def update_form_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    current_version = get_object_or_404(EntryVersion, pk=pk)
    entry = current_version.entry
    version_blocks = current_version.blocks.active().order_by('order')

    if version_blocks.count() == 0:
        version_blocks = [
            EntryVersionBlock.services.factory.create_blank_block(
                entry_version=current_version,
                block_type=BlockTypeChoices.TEXT,
                order=0
            )
        ]

    breadcrumbs = Breadcrumbs()
    breadcrumbs.add_breadcrumb(name='Knowledge')
    breadcrumbs.add_breadcrumb(
        name='Collections',
        href=reverse('django_spire:knowledge:collection:page:list')
    )
    breadcrumbs.add_breadcrumb(
        name=entry.collection.name,
        href=reverse(
            'django_spire:knowledge:collection:page:detail',
            kwargs={'pk': entry.collection_id}
        )
    )
    breadcrumbs.add_breadcrumb(
        name=f'View {entry.name}',
        href=reverse(
            'django_spire:knowledge:entry:version:page:detail',
            kwargs={'pk': entry.current_version_id}
        )
    )
    breadcrumbs.add_breadcrumb(name=f'Edit {entry.name}')

    return portal_views.template_view(
        request,
        page_title=f'Edit {entry.name}',
        page_description=f'Edit {entry.name}',
        breadcrumbs=breadcrumbs,
        context_data={
            'entry': entry,
            'current_version': current_version,
            'version_blocks_json': EntryVersionBlock.services.transformation.objects_to_json(
                objects=version_blocks
            ),
            'version_blocks': version_blocks,
            'block_types': BlockTypeChoices,
        },
        template='django_spire/knowledge/entry/version/page/form_page.html',
    )



import json

from django.conf import settings
from django.http import HttpResponse

from django_spire.ai.chat.message_intel import DefaultMessageIntel
from django_spire.ai.chat.models import Chat
from django_spire.ai.chat.responses import MessageResponseGroup, MessageResponse
from django_spire.ai.chat.choices import MessageResponseType
from django_spire.consts import AI_CHAT_WORKFLOW_SENDER_SETTINGS_NAME


def request_message_render_view(request):
    body_data = json.loads(request.body)

    chat = (
        Chat.objects
        .by_user(request.user)
        .get(id=body_data['chat_id'])
    )

    if chat.is_empty:
        chat.name = body_data['message_body']
        chat.save()

    message_response_group = MessageResponseGroup()

    user_message_response = MessageResponse(
        type=MessageResponseType.REQUEST,
        sender='You',
        message_intel=DefaultMessageIntel(
            text=body_data['message_body']
        )
    )

    message_response_group.add_message_response(
        user_message_response
    )

    chat.add_message_response(user_message_response)

    message_response_group.add_message_response(
        MessageResponse(
            type=MessageResponseType.LOADING_RESPONSE,
            sender='Spire',
            message_intel=DefaultMessageIntel(
                text=body_data['message_body']
            ),
            synthesis_speech=body_data.get('synthesis_speech', False),
        )
    )

    return HttpResponse(
        message_response_group.render_to_html_string(
            context_data={
                "chat_id": chat.id,
                "chat_workflow_name": getattr(settings, AI_CHAT_WORKFLOW_SENDER_SETTINGS_NAME),
            }
        )
    )



from typing import Type

from dandy.llm import BaseLlmMap
from dandy.map import Map

from django_spire.knowledge.collection.models import Collection


def get_entry_map_class(collection: Collection) -> Type[BaseLlmMap]:
    class EntryMap(BaseLlmMap):
        map_keys_description = 'Knowledge Entries'
        map = Map({
            **{
                entry.name: entry
                for entry in collection.entries.all()
              },
            **{'No Matching Knowledge Entries': None}
        })

    return EntryMap



from __future__ import annotations

import shutil

from typing_extensions import TYPE_CHECKING

from django.conf import settings
from django.core.management.base import CommandError

if TYPE_CHECKING:
    from pathlib import Path

    from django_spire.core.management.commands.spire_startapp_pkg.processor import (
        AppTemplateProcessor,
        HTMLTemplateProcessor
    )
    from django_spire.core.management.commands.spire_startapp_pkg.reporter import Reporter


class BaseTemplateManager:
    def __init__(self, base: Path, template: Path):
        self.base = base
        self.template = template

    def _get_destination(self, components: list[str]) -> Path:
        return self.base.joinpath(*components)

    def _report_exists(self, app: str, destination: Path, reporter: Reporter) -> None:
        raise NotImplementedError

    def _report_creating(self, app: str, destination: Path, reporter: Reporter) -> None:
        raise NotImplementedError

    def _report_success(self, app: str, reporter: Reporter) -> None:
        raise NotImplementedError

    def _create_entity(
        self,
        app: str,
        components: list[str],
        processor_method: callable,
        reporter: Reporter,
        missing_template_message: str
    ) -> None:
        destination = self._get_destination(components)

        if destination.exists() and any(destination.iterdir()):
            self._report_exists(app, destination, reporter)
            return

        self._report_creating(app, destination, reporter)
        destination.mkdir(parents=True, exist_ok=True)

        if not self.template.exists():
            raise CommandError(missing_template_message)

        shutil.copytree(
            self.template,
            destination,
            dirs_exist_ok=True,
            ignore=shutil.ignore_patterns('__pycache__', '*.pyc')
        )

        processor_method(destination, components)
        self._report_success(app, reporter)


class AppManager(BaseTemplateManager):
    def get_valid_root_apps(self) -> set[str]:
        return {
            app.split('.')[0]
            for app in settings.INSTALLED_APPS
            if '.' in app and app.split('.')[0] != 'django'
        }

    def is_valid_root_apps(self, components: list[str]) -> None:
        valid = self.get_valid_root_apps()
        root = components[0]

        if root not in valid:
            message = (
                f'Invalid root app "{root}". '
                f'The following are valid root apps: {", ".join(valid)}.'
            )

            raise CommandError(message)

    def validate_app_name_format(self, app: str) -> None:
        if '.' not in app:
            message = (
                'Invalid app name format. '
                'The app path must use dot notation (e.g., "parent.child").'
            )

            raise CommandError(message)

    def parse_app_name(self, app: str) -> list[str]:
        return app.split('.') if '.' in app else [app]

    def get_missing_components(
        self,
        components: list[str],
        registry: list[str]
    ) -> list[str]:
        missing = []
        total = len(components)

        for i in range(total):
            component = '.'.join(components[: i + 1])

            if component not in registry and i > 0:
                missing.append(component)

        return missing

    def _report_exists(self, app: str, destination: Path, reporter: Reporter) -> None:
        reporter.report_app_exists(app, destination)

    def _report_creating(self, app: str, destination: Path, reporter: Reporter) -> None:
        reporter.report_creating_app(app, destination)

    def _report_success(self, app: str, reporter: Reporter) -> None:
        reporter.report_app_creation_success(app)

    def create_custom_app(
        self,
        app: str,
        processor: AppTemplateProcessor,
        reporter: Reporter
    ) -> None:
        components = app.split('.')

        message = (
            f'Template directory "{self.template}" is missing. '
            'Ensure you have a valid app template.'
        )

        self._create_entity(
            app,
            components,
            processor.replace_app_name,
            reporter,
            message
        )


class HTMLTemplateManager(BaseTemplateManager):
    def _report_exists(self, app: str, destination: Path, reporter: Reporter) -> None:
        reporter.report_templates_exist(app, destination)

    def _report_creating(self, app: str, destination: Path, reporter: Reporter) -> None:
        reporter.report_creating_templates(app, destination)

    def _report_success(self, app: str, reporter: Reporter) -> None:
        reporter.report_templates_creation_success(app)

    def create_custom_templates(
        self,
        app: str,
        processor: HTMLTemplateProcessor,
        reporter: Reporter
    ) -> None:
        components = app.split('.')[1:]

        message = (
            f'Template directory "{self.template}" is missing. '
            'Ensure you have a valid templates template.'
        )

        self._create_entity(
            app,
            components,
            processor.replace_template_names,
            reporter,
            message
        )



from django.apps import AppConfig
from django.conf import settings

from django_spire.consts import AI_SMS_CONVERSATION_WORKFLOW_CLASS_SETTINGS_NAME
from django_spire.utils import check_required_apps


class SmsConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'django_spire.ai.sms'
    label = 'django_spire_ai_sms'

    REQUIRED_APPS = ('django_spire_ai',)

    def ready(self):
        if not isinstance(getattr(settings, AI_SMS_CONVERSATION_WORKFLOW_CLASS_SETTINGS_NAME), str):
            raise ValueError(f'"{AI_SMS_CONVERSATION_WORKFLOW_CLASS_SETTINGS_NAME}" must be set in the django settings when using "{self.label}".')

        check_required_apps(self.label)



from typing import Any

from django.core.handlers.wsgi import WSGIRequest

from django_spire.knowledge.collection.models import Collection


def django_spire_knowledge(request: WSGIRequest) -> dict[str, Any]:
    if not request.user.is_authenticated:
        return {}

    return {
        'collection_tree_json': Collection.services.transformation.to_hierarchy_json(
            queryset=Collection.objects.all().select_related('parent').order_by('name'),
            user=request.user,
        )
    }



from __future__ import annotations

from typing_extensions import TYPE_CHECKING

from django.contrib.auth.decorators import permission_required
from django.shortcuts import get_object_or_404, redirect
from django.template.response import TemplateResponse
from django.urls import reverse

from django_spire.core.redirect.safe_redirect import safe_redirect_url
from django_spire.core.shortcuts import get_object_or_null_obj
from django_spire.contrib.form.utils import show_form_errors
from django_spire.history.activity.utils import add_form_activity
from django_spire.contrib.generic_views import portal_views, modal_views

import django_glue as dg

from module import forms, models


if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest


@permission_required('spireparentapp.delete_spirepermission')
def delete_form_modal_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    spirechildapp = get_object_or_404(models.SpireChildApp, pk=pk)

    form_action = reverse(
        'spireparentapp:spirechildapp:delete_form_modal',
        kwargs={'pk': pk}
    )

    def add_activity() -> None:
        spirechildapp.add_activity(
            user=request.user,
            verb='deleted',
            device=request.device,
            information=f'{request.user.get_full_name()} deleted a spirechildapp on "{spirechildapp.spireparentapp}".'
        )

    fallback = reverse(
        'spireparentapp:detail',
        kwargs={'pk': spirechildapp.spireparentapp.pk}
    )

    return_url = safe_redirect_url(request, fallback=fallback)

    return modal_views.dispatch_modal_delete_form_content(
        request,
        obj=spirechildapp,
        form_action=form_action,
        activity_func=add_activity,
        return_url=return_url,
    )


@permission_required('spireparentapp.delete_spirepermission')
def delete_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    spirechildapp = get_object_or_404(models.SpireChildApp, pk=pk)

    return_url = request.GET.get(
        'return_url',
        reverse('spirechildapp:page:list')
    )

    return portal_views.delete_form_view(
        request,
        obj=spirechildapp,
        return_url=return_url
    )


@permission_required('spireparentapp.change_spirepermission')
def form_content_modal_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    spirechildapp = get_object_or_404(models.SpireChildApp, pk=pk)

    dg.glue_model_object(request, 'spirechildapp', spirechildapp)

    context_data = {
        'spirechildapp': spirechildapp
    }

    return TemplateResponse(
        request,
        context=context_data,
        template='spirechildapp/modal/content/form_modal_content.html'
    )


@permission_required('spireparentapp.change_spirepermission')
def form_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    spirechildapp = get_object_or_null_obj(models.SpireChildApp, pk=pk)

    dg.glue_model_object(request, 'spirechildapp', spirechildapp, 'view')

    if request.method == 'POST':
        form = forms.PlaceholderForm(request.POST, instance=spirechildapp)

        if form.is_valid():
            spirechildapp = form.save()
            add_form_activity(spirechildapp, pk, request.user)

            return redirect(
                request.GET.get(
                    'return_url',
                    reverse('spirechildapp:page:list')
                )
            )

        show_form_errors(request, form)
    else:
        form = forms.PlaceholderForm(instance=spirechildapp)

    return portal_views.form_view(
        request,
        form=form,
        obj=spirechildapp,
        template='spirechildapp/page/form_page.html'
    )



from typing import Any

from dandy.llm import BaseLlmBot, Prompt, DefaultLlmIntel

from django_spire.knowledge.entry.models import Entry


class EntrySearchLlmBot(BaseLlmBot):
    @classmethod
    def process(
            cls,
            user_input: str,
            entry: Entry
    ) -> str:
        entry_prompt = Prompt()

        entry_prompt.text('You are a helpful assistant that helps users find information about entries.')

        entry_prompt.text(f'User Input: {user_input}')

        entry_prompt.text(f'Entry: {entry.name}')

        for version_block in entry.current_version.blocks.all():
            entry_prompt.text(version_block.render_to_text())

        result_intel: DefaultLlmIntel = cls.process_prompt_to_intel(
            prompt=entry_prompt
        )

        return result_intel.text


from __future__ import annotations

from module import models

from django_spire.contrib.seeding import DjangoModelSeeder


class SpireChildAppSeeder(DjangoModelSeeder):
     model_class = models.SpireChildApp
     fields = {

     }




from django.contrib.auth.decorators import login_required
from django.shortcuts import get_object_or_404
from django.urls import reverse

from django_spire.auth.permissions.decorators import permission_required
from django_spire.contrib.generic_views import portal_views
from django_spire.help_desk.models import HelpDeskTicket


@permission_required('django_spire_help_desk.delete_helpdeskticket')
def ticket_delete_view(request, pk):
    ticket = get_object_or_404(HelpDeskTicket, pk=pk)

    return portal_views.delete_form_view(
        request=request,
        obj=ticket,
        return_url=request.GET.get(
            'return_url',
            reverse('django_spire:help_desk:page:list')
        )
    )


@login_required()
def ticket_detail_view(request, pk):
    ticket = get_object_or_404(HelpDeskTicket, pk=pk)

    return portal_views.detail_view(
        request=request,
        obj=ticket,
        context_data={
            'ticket': ticket
        },
        template='django_spire/help_desk/page/ticket_detail_page.html',
    )


@login_required()
def ticket_list_view(request):
    tickets = HelpDeskTicket.objects.order_by('-created_datetime').active()

    return portal_views.list_view(
        request=request,
        context_data={
            'tickets': tickets
        },
        model=HelpDeskTicket,
        template='django_spire/help_desk/page/ticket_list_page.html'
    )



from __future__ import annotations

from django_spire.core.middleware.maintenance import MaintenanceMiddleware
from django_spire.core.middleware.profiling import ProfilingMiddleware


__all__ = ['MaintenanceMiddleware', 'ProfilingMiddleware']
